﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.IO;
using ProtoBuf;
using System.Net;
using Utility;

namespace Config
{
    /// <summary>
    /// 服务器条目
    /// </summary>
    [ProtoContract]
    public class ServerItem
    {
        [ProtoMember(1)]
        public string serverno = "";

        [ProtoMember(2)]
        public string desc = "";

        [ProtoMember(3)]
        public string ip = "";

        [ProtoMember(4)]
        public string port = "";

        [ProtoMember(5)]
        public string webip = "";

        [ProtoMember(6)]
        public string webport = "";

        [ProtoMember(7)]
        public string gmip = "";

        [ProtoMember(8)]
        public string gmport = "";
    }

    [ProtoContract]
    public class ServerItems
    {
        [ProtoMember(1)]
        public Dictionary<string, ServerItem> serverMap = new Dictionary<string, ServerItem>();

        //获取实际地址
        public IPEndPoint GetEndPoint(string serverno)
        {
            if (serverMap.ContainsKey(serverno))
            {
                ServerItem item = serverMap[serverno];
                return new IPEndPoint(IPAddress.Parse(item.ip), int.Parse(item.port));
            }

            return null;
        }

        /// <summary>
        /// 获取web监听地址
        /// </summary>
        /// <param name="serverno"></param>
        /// <returns></returns>
        public IPEndPoint GetWebEndPoint(string serverno)
        {
            if (serverMap.ContainsKey(serverno))
            {
                ServerItem item = serverMap[serverno];
                if (item.webip == "" || item.webport == "")
                {
                    return null;
                }
                return new IPEndPoint(IPAddress.Parse(item.webip), int.Parse(item.webport));
            }

            return null;
        }

        /// <summary>
        /// 获取后台监听地址
        /// </summary>
        /// <param name="serverno"></param>
        /// <returns></returns>
        public IPEndPoint GetGMEndPoint(string serverno)
        {
            if (serverMap.ContainsKey(serverno))
            {
                ServerItem item = serverMap[serverno];
                if (item.gmip == "" || item.gmport == "")
                {
                    return null;
                }
                return new IPEndPoint(IPAddress.Parse(item.gmip), int.Parse(item.gmport));
            }

            return null;
        }

        public string GetServerNo(IPEndPoint endpoint)
        {
            foreach (var item in serverMap)
            {
                if (endpoint.Address.ToString() == item.Value.ip && endpoint.Port.ToString() == item.Value.port)
                {
                    return item.Key;
                }
            }

            return "";
        }

        public List<string> GetServerNosByType(string servertype)
        {
            List<string> lstResults = new List<string>();
            foreach (var item in serverMap)
            {
                if (item.Key.ContainsIgnoreCase(servertype))
                {
                    lstResults.Add(item.Key);
                }
            }
            return lstResults;
        }
    }

    [ProtoContract]
    public class LogConfig
    {
        [ProtoMember(1)]
        public bool IsConsoleScreen = false;

        [ProtoMember(2)]
        public bool IsPrintMsg = false;
    }

    [ProtoContract]
    public class ServerConfig
    {
        [ProtoMember(1)]
        public ServerItems serverItems = new ServerItems();

        [ProtoMember(2)]
        public LogConfig logCfg = new LogConfig();

        public string redisIp = "";
        public string redisPort = "";
        public uint HeartSec = 50;

        public void Load(XmlElement rootElem)
        {
            XmlNodeList elementNodes = rootElem.ChildNodes;
            foreach (XmlNode elementNode in elementNodes)
            {
                if (elementNode.Name == "RoutTable")
                {
                    LoadServerIPs((XmlElement)elementNode);
                }
                if (elementNode.Name == "Redis")
                {
                    LoadRedis((XmlElement)elementNode);
                }
                if (elementNode.Name == "LogCfg")
                {
                    LoadLogCfg((XmlElement)elementNode, logCfg);
                }
                if(elementNode.Name == "HeartSec"){
                    XmlElement xe = (XmlElement)elementNode;
                    int sec = int.Parse(xe.GetAttribute("value"));
                    HeartSec = (uint)sec;
                }
            }
        }

        public void LoadServerIPs(XmlElement rootElem)
        {
            serverItems.serverMap.Clear();

            XmlNodeList elementNodes = rootElem.GetElementsByTagName("entry");
            foreach (XmlNode elementNode in elementNodes)
            {
                string serverno = ((XmlElement)elementNode).GetAttribute("serverno");
                string desc = ((XmlElement)elementNode).GetAttribute("desc");
                string ip = ((XmlElement)elementNode).GetAttribute("ip");
                string port = ((XmlElement)elementNode).HasAttribute("port") ? ((XmlElement)elementNode).GetAttribute("port") : "";
                string webip = ((XmlElement)elementNode).HasAttribute("webip") ? ((XmlElement)elementNode).GetAttribute("webip") : "";
                string webport = ((XmlElement)elementNode).HasAttribute("webport") ? ((XmlElement)elementNode).GetAttribute("webport") : "";
                string gmip = ((XmlElement)elementNode).HasAttribute("gmip") ? ((XmlElement)elementNode).GetAttribute("gmip") : "";
                string gmport = ((XmlElement)elementNode).HasAttribute("gmport") ? ((XmlElement)elementNode).GetAttribute("gmport") : "";


                ServerItem item = new ServerItem();
                item.serverno = serverno;
                item.desc = desc;
                item.ip = ip;
                item.port = port;
                item.webip = webip;
                item.webport = webport;
                item.gmip = gmip;
                item.gmport = gmport;

                serverItems.serverMap.Add(serverno, item);
            }
        }

        public void LoadLogCfg(XmlElement rootElem, LogConfig logConfig)
        {
            XmlNodeList elementNodes = rootElem.ChildNodes;
            foreach (XmlNode elementNode in elementNodes)
            {
                if (elementNode.Name == "IsConsoleScreen")
                {
                    logConfig.IsConsoleScreen = bool.Parse(elementNode.InnerText);
                }

                if (elementNode.Name == "IsPrintMsg")
                {
                    logConfig.IsPrintMsg = bool.Parse(elementNode.InnerText);
                }
            }
        }

        public void LoadRedis(XmlElement rootElem)
        {
            redisIp = rootElem.GetAttribute("ip");
            redisPort = rootElem.GetAttribute("port");
        }

        public void Load(string file)
        {
            XmlDocument Doc = new XmlDocument();
            Doc.Load(file);
            XmlElement rootElem = Doc.DocumentElement;
            Load(rootElem);
        }
    }
}